"""
A list of various functions
"""
from .functions import const, lorentz,linear,line,cos_abs,gauss,quadratic,exponential
__all__ = [
    'const', 
    'lorentz',
    'linear',
    'line',
    'cos_abs',
    'gauss',
    'quadratic',
    'exponential'
    ]